@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: driverccid.c 168 2006-03-07 01:33:13Z aquamaniac $
+    cvs         : $Id: driverccid.c 182 2006-05-04 16:04:36Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -30,11 +30,20 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <stdarg.h>
+#include <ctype.h>
+
 
 
 GWEN_INHERIT(LCD_DRIVER, DRIVER_CCID)
 
 
+/* possible values:
+ *   0: undetermined
+ *   1: generic ccid driver pre 1.1.0
+ *  -1: generic ccid driver 1.1.0 or higher
+ */
+static int lc_driver__ccid_pre_1_1_0=0;
+
 
 LCD_DRIVER *DriverCCID_new(int argc, char **argv) {
   DRIVER_CCID *dct;
@@ -682,8 +691,19 @@ GWEN_TYPE_UINT32 DriverCCID_ReaderInfo(LCD_DRIVER *d, LCD_READER *r,
   cnt=bufferlen/sizeof(PCSC_TLV_STRUCTURE);
   tlv=(PCSC_TLV_STRUCTURE*)buffer;
   for (i=0; i<cnt; i++) {
-    DBG_NOTICE(lg, "TLV: %d (%08x)", tlv[i].tag, tlv[i].value);
-    ReaderCCID_SetFeatureCode(r, tlv[i].tag, tlv[i].value);
+    uint32_t v;
+
+    v=tlv[i].value;
+#ifdef LC_ENDIAN_LITTLE
+    /* only translate control codes for generic ccid driver >=1.1.0 */
+    if (lc_driver__ccid_pre_1_1_0==-1)
+      v=((v & 0xff000000)>>24) |
+	((v & 0x00ff0000)>>8) |
+	((v & 0x0000ff00)<<8) |
+	((v & 0x000000ff)<<24);
+#endif
+    DBG_NOTICE(lg, "TLV: %d (%08x)", tlv[i].tag, v);
+    ReaderCCID_SetFeatureCode(r, tlv[i].tag, v);
   }
 
   if (GWEN_Buffer_GetUsedBytes(buf))
@@ -738,6 +758,13 @@ GWEN_TYPE_UINT32 DriverCCID_ConnectReader(LCD_DRIVER *d, LCD_READER *r) {
               "Could not connect any slot");
     return LC_ERROR_NO_SLOTS_CONNECTED;
   }
+
+  /* until now we should have received the debug message from the
+   * reader telling us about the version...
+   */
+  if (lc_driver__ccid_pre_1_1_0==0)
+    /* assume this is a new driver type */
+    lc_driver__ccid_pre_1_1_0=-1;
   return 0;
 }
 
@@ -1019,6 +1046,60 @@ void log_msg(const int priority, const char *fmt, ...) {
     DBG_DEBUG(0, "PCSC: %s", msgBuf);
     break;
   } /* switch */
+  DriverCCID__checkMsg(priority, msgBuf);
+}
+
+
+
+void debug_msg(const int priority, const char *fmt, ...) {
+  char msgBuf[1024];
+  va_list argptr;
+
+  va_start(argptr, fmt);
+  vsnprintf(msgBuf, sizeof(msgBuf), fmt, argptr);
+  va_end(argptr);
+
+  switch(priority) {
+  case 1: /* PCSC_LOG_INFO */
+    DBG_INFO(0, "PCSC: %s", msgBuf);
+    break;
+  case 2: /* PCSC_LOG_ERROR */
+  case 3: /* PCSC_LOG_CRITICAL */
+    DBG_ERROR(0, "PCSC: %s", msgBuf);
+    break;
+  case 0: /* PCSC_LOG_DEBUG */
+  default:
+    DBG_DEBUG(0, "PCSC: %s", msgBuf);
+    break;
+  } /* switch */
+  DriverCCID__checkMsg(priority, msgBuf);
+}
+
+
+
+void log_xxd(const int priority, const char *msg,
+             const unsigned char *buffer,
+	     const int size) {
+  GWEN_LOGGER_LEVEL lv;
+
+  switch(priority) {
+  case 1: /* PCSC_LOG_INFO */
+    lv=GWEN_LoggerLevel_Info;
+    DBG_INFO(0, "PCSC: %s", msg);
+    break;
+  case 2: /* PCSC_LOG_ERROR */
+  case 3: /* PCSC_LOG_CRITICAL */
+    lv=GWEN_LoggerLevel_Error;
+    DBG_ERROR(0, "PCSC: %s", msg);
+    break;
+  case 0: /* PCSC_LOG_DEBUG */
+  default:
+    lv=GWEN_LoggerLevel_Debug;
+    DBG_DEBUG(0, "PCSC: %s", msg);
+    break;
+  } /* switch */
+
+  GWEN_Text_LogString((const char*) buffer, size, 0, lv);
 }
 
 
@@ -1033,3 +1114,62 @@ char *pcsc_stringify_error(long x) {
 }
 
 
+
+void DriverCCID__checkMsg(int priority, const char *msg) {
+  if (lc_driver__ccid_pre_1_1_0==0 && priority>0) {
+    if (GWEN_Text_ComparePattern((const char*)msg,
+                                 "*ProductString: Generic CCID driver v*",
+                                 0)!=-1) {
+      const char *p;
+
+      p=msg;
+      p=strrchr(p, 'v');
+      if (p) {
+        int vmajor=0;
+        int vminor=0;
+        int vpatchlevel=0;
+
+        p++;
+        /* read vmajor */
+        while (*p && isdigit(*p)) {
+          vmajor=(vmajor*10)+(*p-'0');
+          p++;
+        }
+        if (*p=='.') {
+          p++;
+          /* read vminor */
+          while (*p && isdigit(*p)) {
+            vminor=(vminor*10)+(*p-'0');
+            p++;
+          }
+          if (*p=='.') {
+            /* read vpatchlevel */
+            p++;
+            while (*p && isdigit(*p)) {
+              vpatchlevel=(vpatchlevel*10)+(*p-'0');
+              p++;
+            }
+            DBG_NOTICE(0, "Detected Generic CCID driver (%d.%d.%d)",
+                       vmajor, vminor, vpatchlevel);
+
+            /* check whether we have to reverse the translation of the
+             * control codes
+             */
+            if (!(vmajor>1 || (vmajor==1 && vminor>0))) {
+              DBG_WARN(0,
+                       "Old Generic CCID driver, "
+                       "will not translate control codes");
+              lc_driver__ccid_pre_1_1_0=1;
+            }
+            else
+              lc_driver__ccid_pre_1_1_0=-1;
+          }
+        }
+      }
+    } /* if generic ccid driver */
+  } /* if driver type still undetermined */
+}
+
+
+
+
