@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: ctapi.c 115 2005-08-29 10:08:46Z cstim $
+    cvs         : $Id: ctapi.c 186 2006-06-07 13:11:19Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -331,7 +331,7 @@ char CT_data(unsigned short ctn,
 
   DBG_ERROR(CT_API_LOGDOMAIN, "Sending APDU:");
   GWEN_Text_LogString((const char*)command, lenc,
-                      CT_API_LOGDOMAIN, GWEN_LoggerLevelError);
+                      CT_API_LOGDOMAIN, GWEN_LoggerLevelDebug);
   apdu=CTAPI_APDU_new(command, lenc);
   if (!apdu) {
     DBG_ERROR(CT_API_LOGDOMAIN,
@@ -397,7 +397,7 @@ char CT_data(unsigned short ctn,
         }
         else {
           DBG_INFO(CT_API_LOGDOMAIN, "Response received: ");
-          if (GWEN_Logger_GetLevel(CT_API_LOGDOMAIN)>=GWEN_LoggerLevelInfo)
+          if (GWEN_Logger_GetLevel(CT_API_LOGDOMAIN)>=GWEN_LoggerLevelDebug)
             GWEN_Buffer_Dump(rbuf, stderr, 4);
           *sad=*dad;
           memmove(response, GWEN_Buffer_GetStart(rbuf), i);
@@ -428,7 +428,7 @@ char CT_data(unsigned short ctn,
   }
 
   CTAPI_APDU_free(apdu);
-  DBG_ERROR(CT_API_LOGDOMAIN, "CTAPI-Result: %d", result);
+  DBG_INFO(CT_API_LOGDOMAIN, "CTAPI-Result: %d", result);
   if (result==CT_API_RV_OK) {
     GWEN_Text_LogString((const char*)response, *lenr,
                         CT_API_LOGDOMAIN, GWEN_LoggerLevelError);
@@ -663,7 +663,7 @@ char CT__getStatusICC(CTAPI_CONTEXT *ctx,
     *lenr=2;
     return CT_API_RV_OK;
   }
-  else if (res==LC_Client_ResultOk) {
+  else if (res!=LC_Client_ResultOk) {
     return CT_API_RV_ERR_CT;
   }
 
