@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: client.c 178 2006-04-28 22:22:36Z martin $
+    cvs         : $Id: client.c 263 2006-09-19 09:45:45Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -440,24 +440,34 @@ int LC_Client_Walk(LC_CLIENT *cl) {
   LC_SERVER *sv;
   int rv;
   int done;
+  int okCount;
 
   assert(cl);
 
   /* check all servers */
   done=0;
+  okCount=0;
   sv=LC_Server_List_First(cl->servers);
   while(sv) {
     rv=LC_Client_CheckServer(cl, sv);
     if (rv==-1) {
       DBG_INFO(LC_LOGDOMAIN, "Error checking server");
     }
-    else if (rv==0)
-      done++;
+    else {
+      if (rv==0)
+        done++;
+      okCount++;
+    }
     sv=LC_Server_List_Next(sv);
   } /* while */
 
   /* TODO: Check requests for timeouts */
 
+  /* check whether any server succeeded */
+  if (okCount==0) {
+    DBG_ERROR(LC_LOGDOMAIN, "Errors on all servers");
+    return -1;
+  }
   return done?0:1;
 }
 
@@ -871,22 +881,44 @@ int LC_Client_ReadConfigFile(LC_CLIENT *cl, const char *fname){
   found=0;
   buf=GWEN_Buffer_new(0, 256, 0, 1);
   if (!fname) {
+    /* try config file in given data folder */
     GWEN_Buffer_Reset(buf);
     GWEN_Buffer_AppendString(buf, cl->dataDir);
     GWEN_Buffer_AppendString(buf, DIRSEP "chipcardc2.conf");
     f=fopen(GWEN_Buffer_GetStart(buf), "r");
     if (f) {
       fclose(f);
+      DBG_NOTICE(LC_LOGDOMAIN,
+                 "Using configuration file [%s]", GWEN_Buffer_GetStart(buf));
       found=1;
     }
 
     if (!found) {
+      /* then try configuration file in system data folder */
       GWEN_Buffer_Reset(buf);
       GWEN_Directory_OsifyPath(LC_DEFAULT_DATADIR, buf, 1);
       GWEN_Buffer_AppendString(buf, DIRSEP "chipcardc2.conf");
       f=fopen(GWEN_Buffer_GetStart(buf), "r");
       if (f) {
         fclose(f);
+        DBG_NOTICE(LC_LOGDOMAIN,
+                   "Using configuration file [%s]",
+                   GWEN_Buffer_GetStart(buf));
+        found=1;
+      }
+    }
+
+    if (!found) {
+      /* then try example configuration file in system data folder */
+      GWEN_Buffer_Reset(buf);
+      GWEN_Directory_OsifyPath(LC_DEFAULT_DATADIR, buf, 1);
+      GWEN_Buffer_AppendString(buf, DIRSEP "chipcardc2.conf.minimal");
+      f=fopen(GWEN_Buffer_GetStart(buf), "r");
+      if (f) {
+        DBG_WARN(LC_LOGDOMAIN,
+                 "No config file found, using minimal example file [%s]",
+                 GWEN_Buffer_GetStart(buf));
+        fclose(f);
         found=1;
       }
     }
@@ -3438,124 +3470,6 @@ LC_CLIENT_RESULT LC_Client_ReaderCommand(LC_CLIENT *cl,
 
 
 
-
-
-
-
-
-
-
-GWEN_TYPE_UINT32 LC_Client_SendPerformVerification(LC_CLIENT *cl,
-                                                   LC_CARD *cd,
-                                                   const LC_PININFO *pi) {
-  GWEN_DB_NODE *dbReq;
-  GWEN_DB_NODE *dbT;
-  GWEN_TYPE_UINT32 rqid;
-  char numbuf[16];
-
-  dbReq=GWEN_DB_Group_new("Client_Verify");
-  snprintf(numbuf, sizeof(numbuf)-1, "%08x", LC_Card_GetCardId(cd));
-  numbuf[sizeof(numbuf)-1]=0;
-  GWEN_DB_SetCharValue(dbReq, GWEN_DB_FLAGS_OVERWRITE_VARS,
-                       "cardid", numbuf);
-  dbT=GWEN_DB_GetGroup(dbReq, GWEN_DB_FLAGS_DEFAULT, "pinInfo");
-  assert(dbT);
-  LC_PinInfo_toDb(pi, dbT);
-
-  /* send request */
-  rqid=LC_Client_SendRequest(cl, cd, LC_Card_GetServerId(cd), dbReq);
-  if (rqid==0) {
-    DBG_INFO(LC_LOGDOMAIN, "Error sending request");
-    return 0;
-  }
-
-  return rqid;
-}
-
-
-
-LC_CLIENT_RESULT LC_Client_CheckPerformVerification(LC_CLIENT *cl,
-                                                    GWEN_TYPE_UINT32 rid,
-                                                    int *triesLeft){
-  LC_CLIENT_RESULT res;
-  GWEN_DB_NODE *dbRsp;
-  int err;
-  const char *code;
-  const char *text;
-
-  assert(cl);
-  assert(rid);
-  res=LC_Client_CheckResponse(cl, rid);
-  if (res!=LC_Client_ResultOk)
-    return res;
-
-  dbRsp=LC_Client_GetNextResponse(cl, rid);
-  assert(dbRsp);
-
-  err=LC_Client_CheckForError(dbRsp);
-  if (err) {
-    if (err>(int)GWEN_IPC_ERROR_CODES) {
-      DBG_ERROR(LC_LOGDOMAIN, "IPC error %08x", err);
-      GWEN_DB_Group_free(dbRsp);
-      return LC_Client_ResultIpcError;
-    }
-    else {
-      DBG_ERROR(LC_LOGDOMAIN, "Command error %08x", err);
-      GWEN_DB_Group_free(dbRsp);
-      return LC_Client_ResultCmdError;
-    }
-  }
-
-  code=GWEN_DB_GetCharValue(dbRsp, "data/code", 0, "ERROR");
-  text=GWEN_DB_GetCharValue(dbRsp, "data/text", 0, "(none)");
-  *triesLeft=GWEN_DB_GetIntValue(dbRsp, "data/triesLeft", 0,
-                                 *triesLeft);
-  DBG_DEBUG(LC_LOGDOMAIN, "Verify result: %s (%s)", code, text);
-  if (strcasecmp(code, "OK")==0) {
-    res=LC_Client_ResultOk;
-  }
-  else
-    res=LC_Client_ResultGeneric;
-  GWEN_DB_Group_free(dbRsp);
-  return res;
-}
-
-
-
-LC_CLIENT_RESULT LC_Client_PerformVerification(LC_CLIENT *cl,
-                                               LC_CARD *cd,
-                                               const LC_PININFO *pi,
-                                               int *triesLeft) {
-  GWEN_TYPE_UINT32 rqid;
-  LC_CLIENT_RESULT res;
-
-  rqid=LC_Client_SendPerformVerification(cl, cd, pi);
-  if (rqid==0) {
-    DBG_ERROR(LC_LOGDOMAIN, "Could not send request \"PerformVerification\"");
-    return LC_Client_ResultIpcError;
-  }
-  res=LC_Client_CheckResponse_Wait(cl, rqid, cl->longTimeout);
-  if (res!=LC_Client_ResultOk) {
-    if (res==LC_Client_ResultAborted) {
-      DBG_ERROR(LC_LOGDOMAIN, "User aborted");
-      LC_Client_DeleteRequest(cl, rqid);
-    }
-    else {
-      DBG_ERROR(LC_LOGDOMAIN, "No response for request \"PerformVerification\"");
-    }
-    return res;
-  }
-  res=LC_Client_CheckPerformVerification(cl, rqid, triesLeft);
-  if (res!=LC_Client_ResultOk) {
-    DBG_ERROR(LC_LOGDOMAIN, "Error response for request \"PerformVerification\"");
-    return res;
-  }
-
-  return LC_Client_ResultOk;
-}
-
-
-
 void LC_Client_SetHandleInRequestFn(LC_CLIENT *cl,
                                     LC_CLIENT_HANDLE_INREQUEST_FN fn){
   assert(cl);
