@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: notifications.h 61 2005-02-13 04:02:33Z aquamaniac $
+    cvs         : $Id: notifications.h 189 2006-06-16 01:30:37Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -19,6 +19,10 @@
 #include <gwenhywfar/list2.h>
 #include <chipcard2/chipcard2.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 
 typedef struct LC_NOTIFICATION LC_NOTIFICATION;
 
@@ -40,6 +44,9 @@ CHIPCARD_API
 GWEN_DB_NODE *LC_Notification_GetData(const LC_NOTIFICATION *n);
 
 
+#ifdef __cplusplus
+}
+#endif
 
 
 #endif /* CHIPCARD_CLIENT_NOTIFICATIONS_H */
