@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: tlv.h 61 2005-02-13 04:02:33Z aquamaniac $
+    cvs         : $Id: tlv.h 189 2006-06-16 01:30:37Z martin $
     begin       : Sun Jun 13 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -18,6 +18,9 @@
 #include <gwenhywfar/misc.h>
 #include <chipcard2/chipcard2.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 typedef struct LC_TLV LC_TLV;
 
@@ -49,6 +52,9 @@ CHIPCARD_API
 unsigned int LC_TLV_GetTagSize(const LC_TLV *tlv);
 
 
+#ifdef __cplusplus
+}
+#endif
 
 
 #endif /* CHIPCARD_CLIENT_TLV_H */
