@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: monitor_l.h 2 2005-01-02 10:05:37Z aquamaniac $
+    cvs         : $Id: monitor_l.h 188 2006-06-15 20:28:26Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -20,6 +20,9 @@
 #include <chipcard2-client/client/client.h>
 
 
+LCM_MONITOR *LCM_Monitor_new();
+void LCM_Monitor_free(LCM_MONITOR *mm);
+
 int LCM_Monitor_HandleNotification(LCM_MONITOR *mm,
                                    const LC_NOTIFICATION *n);
 
