@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: devicemanager.c 172 2006-03-26 17:49:42Z aquamaniac $
+    cvs         : $Id: devicemanager.c 264 2006-09-19 15:25:03Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -256,14 +256,12 @@ int LCDM_DeviceManager_Init(LCDM_DEVICEMANAGER *dm, GWEN_DB_NODE *dbConfig) {
       LC_DevMonitor_AddScanner(dm->deviceMonitor, scanner);
       scanners++;
     }
-#ifdef USE_PCMCIA
     if (dm->disablePcmciaScan==0) {
       DBG_INFO(0, "Adding PCMCIA bus scanner");
       scanner=LC_PcmciaScanner_new();
       LC_DevMonitor_AddScanner(dm->deviceMonitor, scanner);
       scanners++;
     }
-#endif
     if (dm->disableUsbRawScan==0) {
       DBG_INFO(0, "Adding USB bus scanner");
       scanner=LC_UsbRawScanner_new();
@@ -967,20 +965,8 @@ int LCDM_DeviceManager_CheckDriver(LCDM_DEVICEMANAGER *dm, LCDM_DRIVER *d) {
         DBG_WARN(0, "Driver terminated normally");
         LCDM_Driver_SetProcess(d, 0);
         dst=LC_DriverStatusDown;
-        LCDM_Driver_SetStatus(d, dst);
-        LCS_Server_DriverChg(dm->server,
-                             LCDM_Driver_GetDriverId(d),
-                             LCDM_Driver_GetDriverType(d),
-                             LCDM_Driver_GetDriverName(d),
-                             LCDM_Driver_GetLibraryFile(d),
-                             dst,
-                             "Driver terminated normally");
-        if (LCDM_Driver_GetIpcId(d)) {
-          /* remove IPC node */
-          GWEN_IpcManager_RemoveClient(dm->ipcManager,
-                                       LCDM_Driver_GetIpcId(d));
-          LCDM_Driver_SetIpcId(d, 0);
-        }
+        LCDM_DeviceManager_AbandonDriver(dm, d, dst,
+                                         "Driver terminated normally");
         done++;
       }
       else if (pst==GWEN_ProcessStateAborted) {
@@ -1750,8 +1736,8 @@ void LCDM_DeviceManager_DriverIpcDown(LCDM_DEVICEMANAGER *dm,
                LCDM_Driver_GetDriverName(d),
                LCDM_Driver_GetDriverId(d));
     LCDM_Driver_SetIpcId(d, 0);
-    LCDM_DeviceManager_AbandonDriver(dm, d, LC_DriverStatusDown,
-				     "Driver connection went down");
+    /*LCDM_DeviceManager_AbandonDriver(dm, d, LC_DriverStatusDown,
+				     "Driver connection went down");*/
   }
 }
 
@@ -2000,6 +1986,13 @@ int LCDM_DeviceManager_HandleDriverReady(LCDM_DEVICEMANAGER *dm,
 
     DBG_ERROR(0, "Error in driver \"%08x\": %s",
               driverId, text);
+
+    /* remove request before removing the IPC client */
+    if (GWEN_IpcManager_RemoveRequest(dm->ipcManager, rid, 0)) {
+      DBG_ERROR(0, "Could not remove request");
+      abort();
+    }
+
     ebuf=GWEN_Buffer_new(0, 256, 0, 1);
     GWEN_Buffer_AppendString(ebuf, "Driver error (");
     GWEN_Buffer_AppendString(ebuf, text);
@@ -2008,10 +2001,6 @@ int LCDM_DeviceManager_HandleDriverReady(LCDM_DEVICEMANAGER *dm,
                                      LC_DriverStatusAborted,
                                      GWEN_Buffer_GetStart(ebuf));
     GWEN_Buffer_free(ebuf);
-    if (GWEN_IpcManager_RemoveRequest(dm->ipcManager, rid, 0)) {
-      DBG_ERROR(0, "Could not remove request");
-      abort();
-    }
     return -1;
   }
 
@@ -2856,8 +2845,9 @@ int LCDM_DeviceManager_DeviceUp(LCDM_DEVICEMANAGER *dm,
     GWEN_Buffer_AppendByte(nbuf, '-');
     GWEN_Buffer_AppendString(nbuf, rtype);
     LCDM_Reader_SetReaderName(r, GWEN_Buffer_GetStart(nbuf));
-    DBG_NOTICE(0, "AUTOCONFIG: Created new reader \"%s\" (%04x/%04x)",
-	       GWEN_Buffer_GetStart(nbuf),
+    DBG_NOTICE(0, "AUTOCONFIG: Created new reader \"%s\" (%s/%04x/%04x)",
+               GWEN_Buffer_GetStart(nbuf),
+               LC_Device_BusType_toString(LC_Device_GetBusType(ud)),
 	       LC_Device_GetVendorId(ud),
 	       LC_Device_GetProductId(ud));
     GWEN_Buffer_free(nbuf);
