@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: servicemanager.c 139 2005-11-22 22:39:52Z aquamaniac $
+    cvs         : $Id: servicemanager.c 265 2006-09-19 15:40:15Z martin $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -390,7 +390,7 @@ int LCSV_ServiceManager_CheckService(LCSV_SERVICEMANAGER *svm,
 
   sflags=LCSV_Service_GetFlags(sv);
 
-  DBG_ERROR(0, "Checking service %s (%d)",
+  DBG_DEBUG(0, "Checking service %s (%d)",
             LCSV_Service_GetServiceName(sv), st);
 
   if ((sflags & LC_SERVICE_FLAGS_CLIENT) &&
